import base64
import mailchimp_transactional as MailchimpTransactional
from .init_creds import mailchimp_key, test
from mailchimp_transactional.api_client import ApiClientError
import iso8601
import typing
from .dates import today_argentina
from .init_creds import init_mongo
from .format import muvi_print

mailchimp = MailchimpTransactional.Client(mailchimp_key)

def send_mail_with_attachment(files_attachments:list, receiver_mail, test_mail="matias@muvinai.com",global_vars:list=list()):
    print("Enviando mail facturacion...")
    attachment = []
    try:
        for file in files_attachments:
            with open(file['path'], 'rb') as f:
                file_str = f.read()
                file_str = base64.b64encode(file_str)
                file_str = file_str.decode('utf-8')
            muvi_print('info', 'Archivo adjunto procesado con exito')
            attachment.append({'content': file_str, 'name': file['name'].split('/')[-1], 'type': 'application/pdf'})
    except:
        return 'error: el archivo no pudo ser encontrado o procesado'
    
    if test:
        to_mail = [{"email": test_mail}]
    else:
        to_mail = [{"email": receiver_mail}]
    
    msg = {
        'from_email': 'no-responder@sportclub.com.ar',
        'from_name': 'SportClub',
        'to': to_mail,
        'global_merge_vars': global_vars,
        'attachments': attachment
    }
    try:
        response = mailchimp.messages.send_template(
            {'template_name': 'facturacion', 'template_content': [], 'message': msg})
        print(response)
        return response[0]
    except ApiClientError as error:
        print('An exception occurred: {}'.format(error.text))
        return 'error'


def send_mail_to_template(receiver: dict, template: str, owner, plan_name: str = "None") -> dict:
    """ Enviar mail con un template determinado

    :param receiver: objeto de cliente del destinatario
    :type receiver: dict
    :param template: nombre del template
    :type template: str
    :param plan: nombre del plan, defaults to "None"
    :type plan: str, optional
    :return: informacion del mail
    :rtype: dict
    """
    try:
        sdate = receiver["last_subscription_date"].strftime("%d/%m/%Y")

    except AttributeError:
        sdate = iso8601.parse_date(receiver["last_subscription_date"]).strftime("%d/%m/%Y")
    except:
        return {}

    global_vars = [{"name": "nombre", "content": receiver["nombre"]},
                   {"name": "apellido", "content": receiver["apellido"]},
                   {"name": "documento", "content": receiver["documento"]},
                   {"name": "plan", "content": plan_name},
                   {"name": "fecha_subscripcion", "content": sdate}
                   ]
    if owner["brand_name"] != "SportClub":
        global_vars.extend([
            {"name": "logo", "content": owner["mail_logo"]},
            {"name": "brand_name", "content": owner["brand_name"]}
        ])
    send_mail(receiver["email"], global_vars, template, owner)


def send_alert(reciever_mail: str, proceso: str, mensaje: str, referencia: str, test_mail="matias@muvinai.com") -> typing.Union[dict, None]:
    """ Enviar mensaje de alerta a ignacio@muvinai.com
    :param reciever_mail: email de quien recibe la alerta
    :type reciever_mail: str
    :param proceso: nombre del proceso
    :type proceso: str
    :param mensaje: mensaje
    :type mensaje: str
    :param referencia: referencia
    :type referencia: str
    :return: Respuesta de mailchimp o None en caso de error
    :rtype: dict | None
    """

    global_vars = [{"name": "proceso", "content": proceso},
                   {"name": "mensaje", "content": mensaje},
                   {"name": "referencia", "content": referencia}
                   ]
    owner = {"mail_sender_address": "no-responder@sportclub.com.ar",
            "mail_sender_name": "SportClub"}
    return send_mail(reciever_mail, global_vars, "alertas", owner, test_mail)


def send_mail_inactivo(receiver, owner):
    """ Enviar mail indicando al cliente que está inactivo.

        :param receiver: documento de cliente del destinatario
        :type receiver: dict
        :return: informacion del mail
        :rtype: dict
        """

    global_vars = [{"name": "nombre", "content": receiver["nombre"]}]
    if owner["brand_name"] == "SportClub":
        template = "inactivo"
    elif owner["brand_name"] == "Aranceles":
        return
    else:
        template = "inactivo-nosc"
        global_vars.extend([
            {"name": "logo", "content": owner["mail_logo"]},
            {"name": "brand_name", "content": owner["brand_name"]},
            {"name": "email_contacto", "content": owner["email_contacto"]}
        ])
    return send_mail(receiver["email"], global_vars, template, owner)


def send_mail(receiver_mail, params, template, owner, test_mail="ignacio@muvinai.com"):
    """ Estructura y envía mail

    :param receiver_mail: mail del receptor
    :type receiver_mail: str
    :param params: lista de objetos que son parámetros a pasar al template
    :type params: list
    :param template: nombre del template
    :type template: str
    :param test_mail: mail del receptor en caso de test
    :type receiver_mail: str
    :return: informacion del mail
    :rtype: dict
    """
    print("Enviando mail" + template)

    msg = {
        "from_email": owner["mail_sender_address"],
        "from_name": owner["mail_sender_name"],
        "to": [{"email": test_mail}] if test else [{"email": receiver_mail}],
        "global_merge_vars": params}

    try:
        response = mailchimp.messages.send_template(
            {"template_name": template, "template_content": [], "message": msg})
        print(response)
        return response[0]
    except ApiClientError as error:
        print("An exception occurred: {}".format(error.text))
        return {}


def send_mail_cambio_tarjeta(receiver, owner):
    """ Enviar mail indicando al cliente que debe cambiar la tarjeta.

            :param receiver: documento de cliente del destinatario
            :type receiver: dict
            :return: informacion del mail
            :rtype: dict
            """
    fecha_vigencia = receiver["fecha_vigencia"].strftime("%d/%m/%Y")
    global_vars = [{"name": "nombre", "content": receiver["nombre"]},
                   {"name": "fecha_vigencia", "content": fecha_vigencia},
                   {"name": "email_contacto", "content": owner["email_contacto"]}
                   ]

    if owner["brand_name"] == "SportClub":
        template = "pago-rechazado"
    elif owner["brand_name"] == "Aranceles":
        return
    elif owner["brand_name"].upper() == "AON":
        template = "cambio_de_tarjeta_aon"
        global_vars = [
            {'name':'nombre','content':receiver['name']},
        ]
    else:
        template = "pago-rechazado-nosc"
        global_vars.extend([
            {"name": "horizontal_white", "content": owner["horizontal_white"]},
            {"name": "image_dark", "content": owner["image_dark"]},
            {"name": "image_light", "content": owner["image_light"]},
            {"name": "brand_name", "content": owner["brand_name"]},
            {"name": "email_contacto", "content": owner["email_contacto"]}
        ])

    return send_mail(receiver["email"], global_vars, template, owner)


def send_mail_bienvenida(receiver, plan, owner, test_mail="ignacio@muvinai.com"):
    """ Enviar mail de bienvenida a la suscripción.

            :param receiver: documento de cliente del destinatario
            :type receiver: dict
            :return: informacion del mail
            :rtype: dict
            """

    global_vars = [{"name": "nombre", "content": receiver["nombre"]},
                   {"name": "apellido", "content": receiver["apellido"]},
                   {"name": "documento", "content": receiver["documento"]},
                   {"name": "plan", "content": plan["name"]},
                   {"name": "fecha_subscripcion", "content": today_argentina().strftime("%d/%m/%Y")},
                   ]

    if plan["sede_local"]:
        db = init_mongo()
        sede = db.club.find_one({"_id": plan["sede_local"]})
        global_vars.extend([
            {"name": "direccion", "content": sede["direccion"]},
            {"name": "telefono", "content": sede["telefono"]},
            {"name": "contact-email", "content": sede["contact-email"]}
        ])
        if "instagram" in sede.keys():
            global_vars.append({"name": "instagram", "content": sede["instagram"]})

    if owner["brand_name"] == "SportClub":
        if plan["nivel_de_acceso"] == "Full" or plan["nivel_de_acceso"] == "Flex":
            template = "workclub-bienvenida"
        elif plan["nivel_de_acceso"] == "Local":
            template = "bienvenida-local"
        else:
            template = "bienvenida"
    else:
        template = "bienvenida-nosc"
        global_vars.extend([
            {"name": "logo_mails", "content": owner["mail_logo"]},
            {"name": "brand_name", "content": owner["brand_name"]},
            {"name": "email_contacto", "content": owner["email_contacto"]}
        ])

    return send_mail(receiver["email"], global_vars, template, owner, test_mail)


def send_mail_exitoso(receiver, plan, owner):
    global_vars = [{"name": "nombre", "content": receiver["nombre"]},
                   {"name": "apellido", "content": receiver["apellido"]},
                   {"name": "plan", "content": plan["name"]},
                   {"name": "fecha_subscripcion", "content": today_argentina().strftime("%d/%m/%Y")}
                   ]

    if owner["brand_name"] == "SportClub":
        template = "exitoso"
    else:
        template = "exitoso-nosc"
        global_vars.extend([
            {"name": "logo_mails", "content": owner["mail_logo"]},
            {"name": "brand_name", "content": owner["brand_name"]},
            {"name": "email_contacto", "content": owner["email_contacto"]}
        ])

    return send_mail(receiver["email"], global_vars, template, owner)


def send_mail_checkout_rechazado(receiver, owner, plan, brand):
    global_vars = [
        {"name": "nombre", "content": receiver["nombre"]},
        {"name": "slug", "content": plan["slug"]},
        {"name": "email_contacto", "content": brand["email_contacto"]}
    ]

    if owner["brand_name"] == "SportClub":
        template = "checkout-rechazado"
    else:
        template = "checkout-rechazado-nosc"
        global_vars.extend([
            {"name": "logo_mails", "content": owner["mail_logo"]},
            {"name": "brand_name", "content": owner["brand_name"]},
            {"name": "email_contacto", "content": owner["email_contacto"]}
        ])

    return send_mail(receiver["email"], global_vars, template, owner)


def send_mail_pago_en_efectivo(receiver, owner, final_price, plan_name):
    """ Estructura y envía mail
    :param receiver_mail: mail del receptor
    :type receiver_mail: str
    :param test_mail: mail del receptor en caso de test
    :type receiver_mail: str
    :return: informacion del mail
    :rtype: dict
    """
    global_vars = [
        {
            "name": "nombre",
            "content": receiver["nombre"],
        }, {
            "name": "precio_total",
            "content": final_price
        }, {
            "name": "plan",
            "content": plan_name
        }
    ]
    if owner["brand_name"] == "SportClub":
        template = "nueva-boleta"
    else:
        template = "nueva-boleta-nosc"
        global_vars.extend([
            {"name": "logo", "content": owner["mail_logo"]},
            {"name": "brand_name", "content": owner["brand_name"]},
            {"name": "email_contacto", "content": owner["email_contacto"]}
        ])
    return send_mail(receiver["email"], global_vars, template, owner)


def send_mail_pago_pendiente(receiver, owner, test_mail="ignacio@muvinai.com"):
    """ Estructura y envía mail
        :param receiver_mail: mail del receptor
        :type receiver_mail: str
        :param test_mail: mail del receptor en caso de test
        :type receiver_mail: str
        :return: informacion del mail
        :rtype: dict
        """

    global_vars = [{"name": "nombre", "content": receiver["nombre"]}]
    if owner["brand_name"] == "SportClub":
        template = "pago-pendiente"
    else:
        template = "pago-pendiente-nosc"
        global_vars.extend([
            {"name": "logo", "content": owner["mail_logo"]},
            {"name": "brand_name", "content": owner["brand_name"]},
            {"name": "email_contacto", "content": owner["email_contacto"]}
        ])
    return send_mail(receiver["email"], global_vars, template, owner)


def send_mail_cambio_tarjeta_aon(receiver, owner):
    """ Enviar mail indicando al cliente que debe cambiar la tarjeta.

            :param receiver: documento de cliente del destinatario
            :type receiver: dict
            :return: informacion del mail
            :rtype: dict
            """
    template = "cambio-de-tarjeta-aon"
    global_vars = [
        {'name':'nombre','content':receiver['name']},
    ]

    return send_mail(receiver["email"], global_vars, template, owner)


def send_mail_vencimiento_anual(receiver, owner):
    """ Enviar mail indicando al cliente que debe cambiar la tarjeta.

            :param receiver: documento de cliente del destinatario
            :type receiver: dict
            :return: informacion del mail
            :rtype: dict
            """
    template = "vencimiento-anual"
    global_vars = [
        {"name": "nombre", "content": receiver["nombre"]},
        {"name": "apellido", "content": receiver["apellido"]},
        {"name": "plan", "content": receiver["plan"]["name"]},
        {"name": "fecha_vigencia", "content": receiver["fecha_vigencia"].strftime("%d/%m/%Y")},
    ]
    return send_mail(receiver["email"], global_vars, template, owner)


def send_mail_cambio_plan_herenecia(receiver, owner):
    """ Estructura y envía mail
        :param receiver_mail: mail del receptor
        :type receiver_mail: str
        :param test_mail: mail del receptor en caso de test
        :type receiver_mail: str
        :return: informacion del mail
        :rtype: dict
        """
    template = "vencimiento-anual-herencia"
    global_vars = [
        {"name": "nombre", "content": receiver["nombre"]},
        {"name": "apellido", "content": receiver["apellido"]},
        {"name": "precio_mensual", "content": receiver["plan"]['price']},
        {"name": "plan", "content": receiver["plan"]["name"]},
        {"name": "plan_herencia", "content": receiver["plan_herencia"]["name"]},
        {"name": "fecha_vigencia", "content": receiver["fecha_vigencia"].strftime("%d/%m/%Y")},
        {"name": "precio_plan_herencia", "content": receiver['plan_herencia']['price']},
        {"name": "frecuencia", "content": receiver['plan_herencia']['cobro'].lower()}
    ]
    return send_mail(receiver["email"], global_vars, template, owner)



from bson import ObjectId
from dateutil.relativedelta import *

from ..utilities.dates import (calculate_payment_date, get_periodo,
                               set_next_vigency, today_argentina)
from ..utilities.members import create_history_event


class Client():

    def __init__(self):
        pass

    def create(
        self,
        active_plan_id,
        payment_id,
        card=None,
        discount=None,
        plan_corporativo=None,
        **kwargs
    ):

        self.active_plan_id = ObjectId(active_plan_id)
        self.payment_ids = [payment_id]
        self.last_payment_id = payment_id
        self.plan_corporativo = plan_corporativo
        self.nacimiento = '22/02/2022'
        self.preferred_payment_method = 'tarjeta'
        self.status = 'pago en proceso'
        self.cobros_recurrentes = 0

        self.__dict__.update(kwargs)
        self.discounts = [] if not discount else [discount]
        self.cards = [] if not card else [card]

        if card:
            self.active_card = card['id']

        today = today_argentina()
        f_vigencia_apto = today + relativedelta(days=30)
        self.apto_medico = {
            'url': '',
            'status': 'validado',
            'fecha_vigencia': f_vigencia_apto.replace(day=f_vigencia_apto.day, hour=23, minute=59, second=59)
        }

        self.history = [create_history_event(
            member=self.__dict__, event_type='alta', source='checkout')]

    def set_dates(self, cobro):
        today = today_argentina()
        npd = calculate_payment_date(today.day, cobro, get_periodo(today))
        self.next_payment_date = npd
        self.fecha_vigencia = None if self.status == 'pago en proceso' else set_next_vigency(
            npd)
        self.last_subscription_date = None if self.status == 'pago en proceso' else today
        self.period_init_day = today.day

    def import_from_db(self, client: dict):
        for key in client.keys():
            setattr(self, key, client[key])

    def push_to_db(self, db):
        client = db.clientes.find_one({'documento': self.documento, 'brand_name': self.brand_name})
        if not client:
            inserted = db.clientes.insert_one(self.__dict__)
            self._id = inserted.inserted_id
        else:
            db.clientes.update_one({'documento': self.documento, 'brand_name': self.brand_name}, {
                                   '$set': self.__dict__})

    def to_dict(self):
        return self.__dict__



# Crea o modifica usuarios en sportaccess
import requests
import json
from .init_creds import init_mongo, test
from datetime import datetime
from .projections import proj_accesso_cliente
from pymongo import ReturnDocument
from .format import datetime_parser, fix_bad_characters
from muvinai.utilities.dates import localize

db = init_mongo()

zonas = {
    "Elite": "-1",
    "Total": "1",
    "Plus": "2",
    "Local": "7"
}

sedes = {
    "CORP": "1",
    "ECOM": "900",
    "ALIA": "901",
    "WORK": "1010",
    "MUVI": "900"
}

all_clubes = db.club.find({"sportaccess_id": {"$exists": True}})


# sede_map = {c["sportaccess_id"]: c["name"] for c in all_clubes}
# additional_map = {
#    1: "Corporate",
#    900: "e-Commerce",
#    901: "Alianzas"
# }
# sede_map.update(additional_map)


def crear_usuario_sportclub(cliente: dict, payment_id: int = -1, nivel_de_acceso: str = 'buscar_plan') -> dict:
    """ Crear un usuario en la base de datos de Sportclub

    :param cliente: cliente de mongodb
    :type cliente: dict
    :param payment_id: id del pago de mercadopago
    :type payment_id: str
    :param nivel_de_acceso: nivel de acceso del plan
    :type nivel_de_acceso: str
    :return: los valores posibles son: Elite, Total, Plus, Local, Flex
    :rtype: dict
    """
    db = init_mongo()
    url = "https://apisami.sportclubaccess.com/2.0/socio?"

    plan = db.planes.find_one({"_id": cliente["active_plan_id"]})
    if nivel_de_acceso == 'buscar_plan':
        try:
            nivel_de_acceso = plan['nivel_de_acceso']
        except:
            nivel_de_acceso = 'Total'
    try:
        if plan["sede_local"] is None:
            sede_value = sedes[cliente["sportaccess_id"][0:4]]
        else:
            club = db.club.find_one({"_id": plan["sede_local"]})
            sede_value = club["sportaccess_id"]
    except:
        sede_value = "1"

    if payment_id == -1:
        try:
            payment_id = cliente['payment_ids'][-1]
        except:
            payment_id = 11

    for a in cliente["domicilio"]:
        if not a:
            cliente["domicilio"][a] = "n/a"

    if "nacimiento" not in cliente.keys():
        cliente["nacimiento"] = "22/02/2022"
    elif not cliente["nacimiento"] or cliente["nacimiento"] == "n/a":
        cliente["nacimiento"] = "22/02/2022"

    if "celular" not in cliente.keys():
        cliente["celular"] = "n/a"
    elif not cliente["celular"]:
        cliente["celular"] = "n/a"

    parameters = {
        "username": "3rdPty2902-8891-20",
        "password": "HWXSroa76FSI226JeEtOd9SqDoDXCkGtJyXTk2qEAN2HI34bOVe",
        "nro_socio": cliente["sportaccess_id"],
        "nombre": cliente["nombre"],
        "apellido": cliente["apellido"],
        "documento": cliente["documento"],
        "email": cliente["email"],
        "sexo": "X",
        "nacimiento": cliente["nacimiento"].replace("/", "-"),
        "celular": cliente["celular"],
        "calle": cliente["domicilio"]["calle"],
        "numero": cliente["domicilio"]["altura"],
        "localidad": cliente["domicilio"]["localidad"],
        "cp": cliente["domicilio"]["código postal"],
        "ciudad": cliente["domicilio"]["localidad"],
        "provincia": cliente["domicilio"]["provincia"],
        "nacionalidad": "Argentino",
        "sede": sede_value,
        "zona": "7" if nivel_de_acceso == "Flex" else zonas[nivel_de_acceso],
        "mp_payment_id": int(payment_id),
        "vigencia": cliente["fecha_vigencia"].strftime("%d-%m-%Y")

    }

    lista = [key + "=" + str(parameters[key]) for key in parameters.keys()]
    querystrings = "&".join(lista)
    if test:
        print("Se saltea el post a sportaccess por ser test.")
        return {"status": True, "message": "No se llamó al servicio por ser TEST."}
    r = requests.post(url + querystrings)
    response = r.json()
    if not response["status"]:
        print("Falló la creación de usuario en bd sporclub")
        print(response["message"])
    else:
        print("Usuario creado satisfactoriamente en bd sportclub")

    return response


def consulta_socio(documento: str) -> dict:
    """ Obtener un socio de la base de datos de sportclub
great
    :param documento: numero de documento del socio que buscado
    :type documento: str
    :return: respuesta de SpAccess (puede ser un socio o un mensaje de error)
    :rtype: dict
    """
    url = "https://apisami.sportclubaccess.com/2.0/members?Documento="
    r = requests.get(url + documento)
    r = r.json()
    try:  # Intento corregir los malos caracteres que puedan tener
        r['data']['Apellido'] = fix_bad_characters(r['data']['Apellido'])
        r['data']['Nombre'] = fix_bad_characters(r['data']['Nombre'])
        r['data']['Direccion'] = fix_bad_characters(r['data']['Direccion'])
    except:
        pass
    # Intenta pasar a datetime la fecha
    try:
        r['data']['Vigencia'] = datetime.strptime(r['data']['Vigencia'], '%d/%m/%Y')
        r['data']["FechaIngreso"] = datetime.strptime(r['data']["FechaIngreso"], '%d/%m/%Y')
    except:
        pass
    # Intenta convertir el apto medico
    try:
        r['data']['AptoMedico'] = datetime.strptime(r['data']['AptoMedico'], '%d/%m/%Y')
    except:
        pass
    return r


def consulta_socio_x_nro_socio(nro_socio: str) -> dict:
    """ Obtener un socio de la base de datos de sportclub

    :param documento: numero de documento del socio que buscado
    :type documento: str
    :return: respuesta de SpAccess (puede ser un socio o un mensaje de error)
    :rtype: dict
    """
    url = "https://apisami.sportclubaccess.com/2.0/members?NroSocio="
    r = requests.get(url + nro_socio)
    r = r.json()
    try:  # Intento corregir los malos caracteres que puedan tener
        r['data']['Apellido'] = fix_bad_characters(r['data']['Apellido'])
        r['data']['Nombre'] = fix_bad_characters(r['data']['Nombre'])
        r['data']['Direccion'] = fix_bad_characters(r['data']['Direccion'])
    except:
        pass
    try:  # Intenta pasar a datetime la fecha
        r['data']['Vigencia'] = datetime.strptime(r['data']['Vigencia'], '%d/%m/%Y')
        r['data']["FechaIngreso"] = datetime.strptime(r['data']["FechaIngreso"], '%d/%m/%Y')
    except:
        pass
    # Intenta convertir el apto medico
    try:
        r['data']['AptoMedico'] = datetime.strptime(r['data']['AptoMedico'], '%d/%m/%Y')
    except:
        pass
    return r


def create_external_user(data: dict) -> dict:
    """ Crear un cliente externo en mongodb

    :param data: datos del socio (estos datos se obtienen de la base de sportclub)
    :type data: dict
    :return: datos del socio con los que se guardo en mongodb
    :rtype: dict
    """

    inv_zonas = {a: b for b, a in zonas.items()}
    na = "Elite" if data["Documento"] in strelites else inv_zonas[str(data["Zona"])]

    nuevo_socio = {
        "sportaccess_id": data["NroSocio"],
        "nivel_de_acceso": na,
        "nombre": data["Nombre"],
        "apellido": data["Apellido"],
        "documento": data["Documento"],
        "email": data["Mail"],
        "domicilio": {
            "calle": data["Direccion"],
            "altura": data["Numero"],
            "apto_lote": data["Piso"],
            "localidad": data["Localidad"],
            "provincia": data["Provincia"],
            "código postal": data["CPostal"]
        },
        "celular": data["Movil"],
        "status": "externo",
        "fecha_vigencia": data["Vigencia"],
        "nacimiento": data["Nacimiento"],
        "last_subscription_date": data["FechaIngreso"],
        "sede": data["Sede"],
        "sede_sqlid": int(data["Sede-SqlId"]),
        "plan": data["Plan"],
        "active_plan_id": None,
        "apto_medico_externo": data["AptoMedico"],
        "brand_name": 'SportClub'
    }
    socio = db.clientes.update_one({"documento": data["Documento"], "sportaccess_id": {"$exists": True}},
                                   {"$set": nuevo_socio}, upsert=True)
    nuevo_socio["_id"] = socio.upserted_id
    return nuevo_socio


def cambio_de_plan(documento: str):
    """ Actualizar en mongodb y en SpAccess el plan del socio

    :param documento: numero de documento del socio al cual se le quiere cambiar el plan
    :type documento: str
    """
    db = init_mongo()
    cliente = db.clientes.find_one({"documento": documento})
    cliente["active_plan"] = "total-mensual"
    cliente["nivel_de_acceso"] = "Total"
    crear_usuario_sportclub(cliente, cliente["last_payment_id"], cliente["nivel_de_acceso"])
    db.clientes.update_one({"documento": documento}, {"$set": cliente})


def update_external_user(data: dict):
    """ Actualizar cliente externo

    :param data: datos que se van a actualizar (tienen que estar las keys: Documento, \
                    Zona, NroSocio, Nombre, Apellido, Documento, Mail, Direccion, Numero, Piso, Localidad,\
                    Provincia, CPostal, Movil, Vigencia, Nacimiento, FechaIngreso)
    :type data: dict
    """
    inv_zonas = {a: b for b, a in zonas.items()}
    na = "Elite" if data["Documento"] in strelites else inv_zonas[str(data["Zona"])]

    socio = {
        "sportaccess_id": data["NroSocio"],
        "nivel_de_acceso": na,
        "nombre": data["Nombre"],
        "apellido": data["Apellido"],
        "documento": data["Documento"],
        "email": data["Mail"],
        "domicilio": {
            "calle": data["Direccion"],
            "altura": data["Numero"],
            "apto_lote": data["Piso"],
            "localidad": data["Localidad"],
            "provincia": data["Provincia"],
            "código postal": data["CPostal"]
        },
        "celular": data["Movil"],
        "status": "externo",
        "fecha_vigencia": data["Vigencia"],
        "nacimiento": data["Nacimiento"],
        "last_subscription_date": data["FechaIngreso"],
        "sede": data["Sede"],
        "sede_sqlid": int(data["Sede-SqlId"]),
        "plan": data["Plan"],
        "active_plan_id": None,
        "apto_medico_externo": data["AptoMedico"],
        "brand_name": 'SportClub'
    }
    return db.clientes.find_one_and_update({"documento": data["Documento"]}, {"$set": socio},
                                           return_document=ReturnDocument.AFTER)


def resolve_external_status(documento):
    _client = db.clientes.find_one({"documento": documento, "email": {"$not": {"$regex": "@migracion"}}})
    if not _client or _client["status"] == "inactivo":
        r = consulta_socio(documento)
        try:
            if r["code"] == 404:
                return "*El DNI ingresado no se encuentra en nuestros registros. " \
                       "Por favor ingresá un DNI válido para continuar."
            elif r["code"] > 299:
                return r["message"]
            else:
                return create_external_user(r["data"])
        except:
            return "El servicio de SportAccess no retorna información válida para este documento."
    elif _client["status"] == "externo":
        r = consulta_socio(documento)
        return update_external_user(r["data"])
    else:
        return _client


def consulta_socio_proclub(documento: str) -> dict:
    """ Obtener un socio de la base de datos de sportclub
great
    :param documento: numero de documento del socio que buscado
    :type documento: str
    :return: respuesta de SpAccess (puede ser un socio o un mensaje de error)
    :rtype: dict
    """
    url = "https://apisocios.sportclub.com.ar:3000/members?Documento="
    headers = {
        "x-api-key": "VHGFDn_2212FFF**/-FFFF",
        "Content-Type": "application/json"
    }
    r = requests.get(url + documento, headers=headers)
    r = r.json()

    try:
        int(r["code"])
    except ValueError:
        return
    if r["code"] >= 400:
        return
    if not r["data"]:
        return

    proclub = r["data"][0]
    if "CredencialSpice" in proclub.keys():
        spice = proclub["CredencialSpice"]
    else:
        spice = None

    # Intentamos conversión a formato perfilsocio
    socio = {"_id": None,
             "fecha_vigencia": localize(datetime.strptime(proclub['Vigencia'], '%d/%m/%Y')),
             "last_subscription_date": localize(datetime.strptime(proclub["FechaIngreso"], '%d/%m/%Y')),
             "apto_medico": {"fecha_vigencia": localize(datetime.strptime(proclub['AptoMedico'], '%d/%m/%Y'))},
             "apellido": proclub["Apellido"],
             "documento": documento,
             "nombre": proclub["Nombre"],
             "plan_details": {
                 "nivel_de_acceso": proclub["nivel_acceso"].lower(),
                 "name": proclub["Plan"],
                 "merchant_id": None,
                 "sede_local": proclub["Sede"]
             },
             "token_spice": spice,
             "seller_merchant": None,
             "externo": True,
             "sede_sqlid": proclub["Sede-SqlId"],
             "proclub_id": proclub["NroSocio"]
             }

    return socio


def update_spice(nro_socio: str, spice) -> dict:
    """ Obtener un socio de la base de datos de sportclub
great
    :param documento: numero de documento del socio que buscado
    :type documento: str
    :return: respuesta de SpAccess (puede ser un socio o un mensaje de error)
    :rtype: dict
    """
    url = "https://apisocios.sportclub.com.ar:3000/members?NroSocio="
    headers = {
        "x-api-key": "VHGFDn_2212FFF**/-FFFF",
        "Content-Type": "application/json"
    }
    body = {"Credencial_spice": spice}
    r = requests.put(url + nro_socio, headers=headers, data=json.dumps(body))
    return r


from datetime import datetime, timedelta
from pprint import pprint
from bson import ObjectId
from ..utilities.dates import today_argentina, DIAS_ANTICIPO_DE_PAGO


class Boleta:
    def __init__(self):
        # self.date_created = None
        # self.status = None
        # self.merchant_id = None
        # self.member_id = None
        # self.tries = []
        pass

    def create(self,
               cliente_id,
               merchant_id,
               plan_id,
               source: str,
               status: str = 'error_not_proccesed',
               charges_detail: dict = dict(),
               opd: datetime = today_argentina(),
               date_created: datetime = today_argentina(),
               additional_fields: dict = dict(),
            ):

        ids = [cliente_id, merchant_id, plan_id]
        for field_id in ids:
            try:
                field_id = ObjectId(field_id)
            except:
                pass
        self.member_id = cliente_id
        self.date_created = date_created
        self.original_payment_date = opd
        self.source = source
        self.tries = list()
        self.status = status
        self.merchant_id = merchant_id
        self.charges_detail = charges_detail
        self.period = None
        # Plan
        self.plan_id = plan_id
        # Si existen campos adicionales a los tradicionales se agregan
        for key, value in additional_fields.items():
            setattr(self, key, value)

    def push_to_db(self, db, id_return=False):
        #db = init_mongo()
        if hasattr(self, '_id'):
            result = db.boletas.update_one(
                {'_id': self._id}, {'$set': self.__dict__})
            if id_return:
                return self._id
        else:
            result = db.boletas.insert_one(self.__dict__)
            self._id = result.inserted_id
            if id_return:
                return result.inserted_id
        pass

    def import_from_db(self, boleta: dict):
        for key in boleta.keys():
            setattr(self, key, boleta[key])

    def add_try(self, payment_data: dict, card_data: dict = dict()):
        # Complete with card data
        if card_data != dict():
            for e in ["card_brand", "card_id", "payment_type"]:
                if e in card_data:
                    payment_data[e] = card_data[e]
                elif e not in "payment_data":
                    payment_data[e] = None

        if 'payment_type' not in payment_data:
            try:
                payment_data['payment_type'] = payment_data['card']['card_type']
            except:
                payment_data['payment_type'] = None
        # Genera el nuevo try
        new_try = {
            'try_number': len(self.tries)+1,
            'payment_day': payment_data['payment_day'],
            'payment_type': payment_data['payment_type'],
            'status': payment_data['status'],
            'status_detail': payment_data['status_detail'],
            'card_id': payment_data['card_id'],
            'card_brand': payment_data['card_brand'],
            'payment_id': payment_data['id'],
            'processor': payment_data['processor'] if 'processor' in payment_data else None
        }
        self.tries.append(new_try)
        self.status = new_try['status']
        pass

    def make_period(self, fecha_de_cobro: datetime, DIAS_DE_ANTICIPO: int = 0):
        if 'recurring' in self.source:
            DIAS_DE_ANTICIPO = DIAS_ANTICIPO_DE_PAGO
        fecha_de_cobro += timedelta(days=DIAS_DE_ANTICIPO)
        period = f"{fecha_de_cobro.month}/{fecha_de_cobro.year}"
        self.period = period

    def print(self):
        pprint(self.__dict__)

    def to_dict(self):
        return self.__dict__



from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography import x509
from cryptography.x509.oid import NameOID
from reportlab.pdfgen import canvas
from reportlab.lib.units import mm,inch
from reportlab.lib.pagesizes import A4
from PyPDF2 import PdfFileWriter, PdfFileReader
import numpy as np
import base64
import json
import qrcode
import locale
import io
from babel.numbers import format_number

from datetime import datetime
from dateutil.relativedelta import relativedelta
from .dates import today_argentina
from .format import muvi_print
from .init_creds import init_mongo
import traceback
from pprint import pprint
from .dates import localize

def generateKey(cuit:str,bits:int=2048,save_file=False):
    """ Funcion para generar una private key con python

    :param cuit: CUIT a generar la PK
    :type cuit: int
    :param bits: Bits de la clave privada (2048 por default)
    :type bits: int
    :param type_pk: tipo de clave, por default TYPE_RSA
    :type type_pk: crypto Type
    :param save_file: si se desea guardar la clave en un archivo, False por default
    :type save_file: boolean
    :return: key
    :rtype: Key de PyOpenSSL

    """

    pk = rsa.generate_private_key(public_exponent=65537, key_size=bits, backend=default_backend)
    
    if save_file:
        keyfile = 'pk_' + cuit + '.key'
        f = open(keyfile, "wb")
        pk_byte = pk.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()
        )
        f.write(pk_byte)
        f.close()

    return pk

def generateCSR(cuit:str,razon_social:str,key,save_file=False):
    """ Funcion para generar el CSR

    :param cuit: CUIT a generar la PK
    :type cuit: int
    :param razon_social: Razon social del facturante
    :type razon_social: str
    :param save_file: si se desea guardar el certificado en un archivo, False por default
    :type save_file: boolean
    :return: CSR
    :rtype: string

    """
    
    CSR = x509.CertificateSigningRequestBuilder().subject_name(x509.Name([
        x509.NameAttribute(NameOID.COMMON_NAME, "muvi_facturacion"),
        x509.NameAttribute(NameOID.COUNTRY_NAME, "AR"),
        x509.NameAttribute(NameOID.ORGANIZATION_NAME, razon_social),
        x509.NameAttribute(NameOID.SERIAL_NUMBER, "CUIT" + cuit)
    ])).sign(key, hashes.SHA256())
    
    if save_file:
        csrfile = f'cert_{str(cuit)}.csr'
        f = open(csrfile, "wb")
        f.write(CSR.public_bytes(serialization.Encoding.PEM))
        f.close()
    return CSR


#### PDF PARA LAS FACTURAS

def format_numero_factura(pto_venta:int,cbte_num:int,tipo_cbte:int,short=False) -> str:
    if np.isnan(pto_venta) or np.isnan(cbte_num) or np.isnan(tipo_cbte):
        return '-'

    # Version corta
    if short:
        return "%03d" % (tipo_cbte,) + '_' + "%05d" % (pto_venta,) + '_' + "%08d" % (cbte_num,)

    # Version larga
    if tipo_cbte == 6:
        nro_factura = 'Factura B - N°' + "%05d" % (pto_venta,) + '-' + "%08d" % (cbte_num,)
    elif tipo_cbte == 8:
        nro_factura = 'Nota de Crédito B - N°' + "%05d" % (pto_venta,) + '-' + "%08d" % (cbte_num,)
    else:
        nro_factura = 'Factura B - N°' + "%05d" % (pto_venta,) + '-' + "%08d" % (cbte_num,)
    return nro_factura

def limit_string(string: str,limit: int) -> str:

    if len(string) > limit:
        return string[:limit]
    else:
        return string

def process_razon_social (rz: str, limit_long: str):
    first_row = ''
    second_row = ''
    if len(rz) > limit_long:
        rz_list = rz.split(' ')
        long_string = 0
        for word in rz_list:
            long_string += len(word)
            if long_string <= limit_long:
                first_row = first_row + word + ' ' 
            else:
                second_row = second_row + word + ' '
    else:
        first_row = rz
    return [first_row, second_row]

def make_qr(factura:dict, qr_path:str):

    url = "https://www.afip.gob.ar/fe/qr/?p=%s"
    
    try:
        documento = int(factura['data_cliente']['documento'])
    except:
        documento = factura['data_cliente']['documento']
    fecha_factura = localize(factura['data_factura']['fecha_factura']) # Se pasa a timezone argentina
    fecha_factura = fecha_factura.strftime("%Y-%m-%d")
    data = {
        'ver': 1,
        'fecha': fecha_factura,
        'cuit': int(factura['data_facturante']['cuit']),
        'ptoVta': int(factura['data_factura']['pto_venta']),
        'tipoCmp': int(factura['data_factura']['tipo_comprobante']),
        'nroCmp': int(factura['data_factura']['num_comprobante']),
        'importe': float(factura['data_venta']['importe_total']),
        'moneda': 'PES',
        'ctz': float(1.000),
        'tipoDocRec': 96, # Por que es DNI
        'nroDocRec': documento,
        'tipoCodAut': 'E',  # CAE o A si CAEA
        'codAut': int(factura['data_cae']['cae'])
    }

    data_json = json.dumps(data)
    url = url % (base64.b64encode(data_json.encode('ascii')).decode('ascii'))

    qr = qrcode.QRCode(
        version=1,
        error_correction=qrcode.constants.ERROR_CORRECT_L,
        box_size=10,
        border=4
    )
    qr.add_data(url)
    qr.make(fit=True)

    img = qr.make_image(fill_color='black', back_color='white')

    img.save(qr_path, "PNG")
    print(f"QR creado en {qr_path}")

def make_factura_pdf(files_names:dict,factura:dict) -> str:
    """
    :param files_names: Nombres de los archivos,
    FILES_NAMES = {
        'QR_PATH':'...',
        'TEMPLATE_FACTURA':'...',
        'TEMPLATE_NOTA_DE_CREDITO':'...',
        'PDF_NAME':'...'}
    :param factura: Dict de factura proveniente de mongo
    :return: Lista con los archivos que genero
    """
    packet = io.BytesIO()
    can = canvas.Canvas(packet)
    x_page = 210
    y_page = 297
    can.setPageSize(A4)

    #### FORMAT DEL PRECIO ################################
    price = '$ ' + format_number(factura['data_venta']['importe_total'],locale='es')
    descuento_price = '$ ' + format_number(0 ,locale='es')

    #### TIPO DE BOLETA ###################################
    can.setFillColorRGB(0.25, 0.25, 0.25)
    can.setFont("Helvetica-Bold", 42)
    can.drawString(99.5 * mm, (y_page - 45) * mm, "B")
    can.setFont("Helvetica-Bold", 12)
    can.drawString(96.5 * mm, (y_page - 52) * mm, "COD. 0"+str(factura['data_factura']['tipo_comprobante']))

    #### DATOS DEL FACTURANTE #############################
    can.setFillColorRGB(1, 1, 1)
    can.setFont("Helvetica-Bold", 11)
    can.drawString(126 * mm, (y_page - 35.2) * mm, format_numero_factura(int(factura['data_factura']['pto_venta']),int(factura['data_factura']['num_comprobante']),int(factura['data_factura']['tipo_comprobante'])))

    can.setFillColorRGB(0, 0, 0)
    can.setFont("Helvetica", 10)
    can.drawString(126 * mm, (y_page - 41.5) * mm, 'Fecha: ' + factura['data_factura']['fecha_factura'].strftime("%Y-%m-%d"))
    cuit = factura['data_facturante']['cuit']
    can.drawString(165 * mm, (y_page - 41.5) * mm, 'CUIT: '+cuit[0:2]+'-'+cuit[2:-1]+'-'+cuit[-1])
    razon_social = process_razon_social(factura['data_facturante']['razon_social'], 25)
    can.drawString(126 * mm, (y_page - 47) * mm, 'Razon Social: ' + razon_social[0])
    can.drawString(126 * mm, (y_page - 52.5) * mm, razon_social[1])
    can.drawString(126 * mm, (y_page - 58) * mm, 'Inicio de Actividades: '+factura['data_facturante']['inicio_de_actividades'].strftime("%Y-%m-%d"))
    can.drawString(126 * mm, (y_page - 63.5) * mm, 'Ingresos Brutos: '+ factura['data_facturante']['ii_bb'])

    can.setFillColorRGB(0.2, 0.2, 0.2)
    can.setFont("Helvetica-Bold", 11)
    can.drawString(8 * mm, (y_page - 36) * mm, 'DATOS DEL FACTURANTE')
    can.setFillColorRGB(0, 0, 0)
    can.setFont("Helvetica", 10)
    direccion_facturante = limit_string(factura['data_facturante']['domicilio']['calle'],45) + ' ' + factura['data_facturante']['domicilio']['altura']
    can.drawString(8 * mm, (y_page - 41.5) * mm, 'Dirección: '+direccion_facturante)
    can.drawString(8 * mm, (y_page - 47) * mm, limit_string(factura['data_facturante']['domicilio']['localidad'],50))
    can.drawString(8 * mm, (y_page - 52.5) * mm, 'Teléfono: ' + factura['data_facturante']['telefono'])
    can.drawString(8 * mm, (y_page - 58) * mm, 'Email: ' + factura['data_facturante']['email'])
    can.drawString(8 * mm, (y_page - 63.5) * mm, 'IVA RESPONSABLE INSCRIPTO')

    #### DATOS DE LAS CONDICIONES DE VENTA ################
    can.setFillColorRGB(0.2, 0.2, 0.2)
    can.setFont("Helvetica-Bold", 10)
    can.drawString(120 * mm, (y_page - 74) * mm, 'CONDICIONES DE VENTA')
    can.setFillColorRGB(0, 0, 0)
    can.setFont("Helvetica", 9)
    if 'payment_type' in factura['data_factura']:
        if 'payment_type' == '':
            metodo_de_pago = 'MercadoPago'
        elif 'payment_type' == 'cash':
            metodo_de_pago = 'Efectivo'
        else:
            metodo_de_pago = '-'
    else:
        metodo_de_pago = '-'
    can.drawString(120 * mm, (y_page - 82) * mm, 'Método de pago:' + metodo_de_pago)
    can.drawString(120 * mm, (y_page - 88) * mm, 'Tipo: Servicios')
    can.drawString(120 * mm, (y_page - 94) * mm, 'Fecha de inicio de servicios: ' + factura['data_venta']['inicio_servicios'].strftime("%Y-%m-%d"))
    can.drawString(120 * mm, (y_page - 100) * mm, 'Fecha de fin de servicios: ' + factura['data_venta']['fin_servicios'].strftime("%Y-%m-%d"))
    can.drawString(120 * mm, (y_page - 106) * mm, 'Fecha de pago del servicio: ' + factura['data_venta']['fecha_pago'].strftime("%Y-%m-%d"))

    #### DATOS DEL CLIENTE ################################
    can.setFillColorRGB(0.2, 0.2, 0.2)
    can.setFont("Helvetica-Bold", 10)
    can.drawString(8 * mm, (y_page - 74) * mm, 'INFORMACIÓN DEL CLIENTE')
    can.setFillColorRGB(0, 0, 0)
    can.setFont("Helvetica", 10)
    cliente_name = limit_string(factura['data_cliente']['nombre'],30) + " " + factura['data_cliente']['apellido']
    can.drawString(8 * mm, (y_page - 80) * mm, 'Cliente: '+limit_string(cliente_name,55))
    direccion_cliente = limit_string(factura['data_cliente']['domicilio']['calle'],45) + ' ' + factura['data_cliente']['domicilio']['altura']
    can.drawString(8 * mm, (y_page - 86) * mm, 'Direccion: '+ limit_string(direccion_cliente,55))
    can.drawString(8 * mm, (y_page - 92) * mm, 'Provincia: ' + factura['data_cliente']['domicilio']['provincia'])
    can.drawString(8 * mm, (y_page - 98) * mm, 'Email: '+factura['data_cliente']['email'])
    can.drawString(8 * mm, (y_page - 104) * mm, 'Documento: '+str(factura['data_cliente']['documento']))
    can.drawString(8 * mm, (y_page - 110) * mm, 'Condición: Consumidor Final')

    #### CONCEPTOS ########################################
    can.setFillColorRGB(0, 0, 0)
    can.setFont("Helvetica", 10)
    # Cantidad
    can.drawString(10 * mm, (y_page - 128.5) * mm, "1,00")
    # Descripcion del Plan
    can.drawString(42 * mm, (y_page - 128.5) * mm, factura['data_plan']['plan_name'])
    # Subtotal
    can.drawString(150 * mm, (y_page - 128.5) * mm,price)
    # Total
    can.drawString(178 * mm, (y_page - 128.5) * mm,price)
    # Si se desea agregar otro item deberia ser en la altura 138.5

    #### PRICES ###########################################
    can.setFillColorRGB(0,0,0)
    can.setFont("Helvetica",11)
    can.drawString(120 * mm, (y_page - 226) * mm, "Subtotal")
    can.drawString(165 * mm, (y_page - 226) * mm, price)
    can.drawString(120 * mm, (y_page - 240) * mm, "Total Descuento")
    can.drawString(165 * mm, (y_page - 240) * mm, descuento_price)
    can.setFillColorRGB(1,1,1)
    can.setFont("Helvetica-Bold",15)
    can.drawString(120 * mm, (y_page-252) * mm, "Final Total")
    can.drawString(165 * mm, (y_page-252) * mm, price)

    #### CAE y QR #########################################
    result_qr = make_qr(factura,files_names['QR_PATH'])
    print(result_qr)
    can.setFillColorRGB(0.1, 0.1, 0.1)
    can.setFont("Helvetica-Bold", 11)
    can.drawString(46 * mm, (y_page-268) * mm, 'N° de CAE: ' + factura['data_cae']['cae'])
    can.drawString(46 * mm, (y_page-278) * mm, 'Fecha de Vencimiento: ' + factura['data_cae']['fecha_vencimiento'].strftime('%d/%m/%Y'))
    can.drawInlineImage(files_names['QR_PATH'], x=15*mm, y=(y_page-284.5)*mm, width=26 * mm, height=26 * mm)

    #### NOTA DE CREDITO ##################################
    if 'data_nota_de_credito' in factura:
        can.setFillColorRGB(0, 0, 0)
        can.setFont("Helvetica", 10)
        can.drawString(8 * mm, (y_page - 226) * mm, 'Nota de crédito B, asociada con:')
        can.drawString(8 * mm, (y_page - 232) * mm, format_numero_factura(pto_venta=factura['data_nota_de_credito']['factura_asociada']['pto_venta'],
                                                                          cbte_num=factura['data_nota_de_credito']['factura_asociada']['num_comprobante'],
                                                                          tipo_cbte=factura['data_nota_de_credito']['factura_asociada']['tipo_comprobante']))

    #### SE GUARDA EL PDF #################################
    can.save()
    packet.seek(0)
    new_pdf = PdfFileReader(packet)
    if 'TEMPLATE_FOLDER' in files_names:
        template_factura = files_names['TEMPLATE_FOLDER']+files_names['TEMPLATE_FACTURA']
        template_nota_de_credito = files_names['TEMPLATE_FOLDER']+files_names['TEMPLATE_NOTA_DE_CREDITO']
    else:
        template_factura = files_names['TEMPLATE_FACTURA']
        template_nota_de_credito = files_names['TEMPLATE_NOTA_DE_CREDITO']
    if factura['data_factura']['tipo_comprobante'] == 6:
        print('TEMPLATE FACTURA PATH:',template_factura)
        existing_pdf = PdfFileReader(open(template_factura, "rb"))
    elif factura['data_factura']['tipo_comprobante'] == 8:
        print('TEMPLATE NOTA DE CREDITO PATH:',template_nota_de_credito)
        existing_pdf = PdfFileReader(open(template_nota_de_credito, "rb"))
    output = PdfFileWriter()
    page = existing_pdf.getPage(0)
    page.mergePage(new_pdf.getPage(0))
    output.addPage(page)
    if 'FOLDER_PDF' in files_names:
        file_name_pdf = files_names['FOLDER_PDF']+files_names['PDF_NAME']
    else:
        file_name_pdf = files_names['PDF_NAME']
    outputStream = open(file_name_pdf, "wb")
    output.write(outputStream)
    outputStream.close()
    return file_name_pdf

def make_name_pdf(factura:dict) -> str:
    name_pdf = str(factura['data_facturante']['cuit'])+'_'+format_numero_factura(int(factura['data_factura']['pto_venta']),
                                                                                 int(factura['data_factura']['num_comprobante']),
                                                                                 int(factura['data_factura']['tipo_comprobante']),
                                                                                 short=True)+'.pdf'
    return name_pdf


#### PROCESO DE FACTURACION --->

def check_afip_data(merchant:dict,list_to_check:list) -> int:
    muvi_print('info','<--- CHECKS AFIP START --->')
    errors = 0
    if 'afip' in merchant:
        for check in list_to_check:
            if check in merchant['afip']:
                muvi_print('successful',f"OK: {check}")
            else:
                errors += 1
                muvi_print('error',f"ERROR: No se encuentra {check}")
        if 'activate_facturacion' in merchant['afip'] and not merchant['afip']['activate_facturacion']:
            muvi_print('error',f"ERROR: El merchant no tiene activado el proceso de facturacion")
            errors += 1
    else:
        errors += 1
        muvi_print('error','ERROR: El merchant no tiene data de AFIP')
    muvi_print('info', '<--- CHECKS AFIP END --->')
    return errors

TIPO_DOCUMENTO = {
    'CUIT': 80,
    'CUIL': 86,
    'CDI': 87,
    'LE': 89,
    'LC': 90,
    'CI_extranjera': 91,
    'acta_nacimiento': 93,
    'pasaporte': 94,
    'DNI': 96,
}

TIPO_COMPROBANTE = {
    'factura_a': 1,
    'nota_debito_a': 2,
    'nota_credito_a': 3,
    'factura_b': 6,
    'nota_debito_b': 7,
    'nota_credito_b': 8,
}

TIPO_CONCEPTO = {
    'producto': 1,
    'servicios': 2,
    'protuctos_servicios': 3
}

TIPO_IVA = {
    21.0:5,         # Iva del 21%
    10.5:4          # Iva del 10.5%
}

def process_period(boleta:dict):
    try:
        pid = int(boleta['cliente']['period_init_day'])
        period = boleta['period'].split('/')
        init_date = datetime(day=pid,month=int(period[0]),year=int(period[1]))
        sum_months = relativedelta(months=0)
        duration_map = {"Mensual": 1, "Trimestral": 3, "Cuatrimestral": 4, "Semestral": 6, "Anual": 12}
        sum_months = relativedelta(months=duration_map[boleta['plan_cobro']])
        end_date = init_date + sum_months
    except:
        init_date = today_argentina()
        end_date = today_argentina()
    return init_date,end_date

def format_factura_afip(boleta: dict,
                        merchant: dict,
                        nro_comprobante: int,
                        tipo_cbte: int,
                        price: int,
                        porcentaje_iva:float,
                        factura_existente:dict) -> dict:
    factura = dict()
    factura['FeCabReq'] = {
        'CantReg': 1,
        'PtoVta': merchant['afip']['pto_venta'],
        'CbteTipo': tipo_cbte
    }

    #### PROCESO EL PERIODO ###############################
    init_serv,end_serv = process_period(boleta)

    #### SE CALCULA EL IVA ################################

    imp_total = round(price,2)
    imp_neto = round(price/(1+porcentaje_iva/100),2)
    imp_iva = round(imp_total - imp_neto,2)
    muvi_print('info',f"Final Price: {price} = IVA: {imp_iva} + NETO: {imp_neto} (Tipo de IVA: {TIPO_IVA[porcentaje_iva]})")

    #######################################################

    factura['FeDetReq'] = {
        'FECAEDetRequest': {
            'Concepto': TIPO_CONCEPTO['servicios'],
            'DocTipo': TIPO_DOCUMENTO['DNI'],
            'DocNro': boleta['cliente']['documento'],
            'CbteDesde': nro_comprobante,
            'CbteHasta': nro_comprobante,
            'CbteFch': today_argentina().strftime("%Y%m%d"),
            'ImpTotal': imp_total,
            'ImpTotConc': 0,
            'ImpNeto': imp_neto,
            'ImpOpEx': 0,
            'ImpTrib': 0,
            'ImpIVA': imp_iva,
            'Iva': [{'AlicIva':{
                    'Id':5,#TIPO_IVA[porcentaje_iva],
                    'BaseImp': imp_neto,
                    'Importe': imp_iva
                }
            }],
            'FchServDesde': init_serv.strftime("%Y%m%d"),
            'FchServHasta': end_serv.strftime("%Y%m%d"),
            'FchVtoPago': today_argentina().strftime("%Y%m%d"),
            'MonId': 'PES',
            'MonCotiz': 1,
        }
    }
    if tipo_cbte == 8:
        factura['FeDetReq']['FECAEDetRequest']['CbtesAsoc'] = [{
            'CbteAsoc': {
                'Tipo':factura_existente['data_factura']['tipo_comprobante'],
                'PtoVta': factura_existente['data_factura']['pto_venta'],
                'Nro': factura_existente['data_factura']['num_comprobante'],
                'Cuit': factura_existente['data_facturante']['cuit'],
                'CbteFch':factura_existente['data_factura']['fecha_factura'].strftime("%Y%m%d")
            }
        }]
    return factura

def format_factura_db(boleta:dict,
                      factura:dict
                      ,response_factura:dict
                      ,data_afip:dict) -> dict:
    if 'payment_type' not in boleta['tries']:
        boleta['tries']['payment_type'] = '-'
    factura_formateada = {
        'data_factura':{
            'tipo_comprobante':factura['FeCabReq']['CbteTipo'],
            'num_comprobante':response_factura['FeDetResp']['FECAEDetResponse'][0]['CbteDesde'],
            'pto_venta':factura['FeCabReq']['PtoVta'],
            'fecha_factura':datetime.utcnow(),
            'mp_payment_id':boleta['tries']['payment_id'],
            'payment_type':boleta['tries']['payment_type'],
            'fecha_real_del_pago':boleta['tries']['payment_day'],
            'source_boleta':boleta['source']
        },
        'data_facturante':{
            'cuit':data_afip['cuit'],
            'razon_social':data_afip['razon_social'],
            'inicio_de_actividades':data_afip['inicio_de_actividades'],
            'ii_bb':data_afip['ii_bb'],
            'domicilio':data_afip['domicilio'],
            'telefono':data_afip['telefono'],
            'email':data_afip['email'],
            'merchant_id': boleta['merchant_id']
        },
        'data_cliente':{
            'nombre':boleta['cliente']['nombre'],
            'apellido':boleta['cliente']['apellido'],
            'domicilio':boleta['cliente']['domicilio'],
            'email':boleta['cliente']['email'],
            'documento':factura['FeDetReq']['FECAEDetRequest']['DocNro']
        },
        'data_venta':{
            'inicio_servicios': datetime.strptime(factura['FeDetReq']['FECAEDetRequest']['FchServDesde'], '%Y%m%d'),
            'fin_servicios':datetime.strptime(factura['FeDetReq']['FECAEDetRequest']['FchServHasta'], '%Y%m%d'),
            'fecha_pago':datetime.strptime(factura['FeDetReq']['FECAEDetRequest']['FchVtoPago'], '%Y%m%d'),
            'importe_total':factura['FeDetReq']['FECAEDetRequest']['ImpTotal'],
            'importe_neto':factura['FeDetReq']['FECAEDetRequest']['ImpNeto'],
            'importe_iva':factura['FeDetReq']['FECAEDetRequest']['ImpIVA'],
        },
        'data_plan':{
            'plan_name':boleta['plan_name']
        },
        'data_cae': {'cae': response_factura['FeDetResp']['FECAEDetResponse'][0]['CAE'],
                    'fecha_vencimiento': datetime.strptime(response_factura['FeDetResp']['FECAEDetResponse'][0]['CAEFchVto'], '%Y%m%d')
        }
    }
    if factura['FeCabReq']['CbteTipo'] == 8:
        factura_formateada['data_nota_de_credito'] = {
            'factura_asociada':{
                'tipo_comprobante':factura['FeDetReq']['FECAEDetRequest']['CbtesAsoc'][0]['CbteAsoc']['Tipo'],
                'num_comprobante': factura['FeDetReq']['FECAEDetRequest']['CbtesAsoc'][0]['CbteAsoc']['Nro'],
                'pto_venta': factura['FeDetReq']['FECAEDetRequest']['CbtesAsoc'][0]['CbteAsoc']['PtoVta'],
                'cuit':factura['FeDetReq']['FECAEDetRequest']['CbtesAsoc'][0]['CbteAsoc']['Cuit'],
                'fecha_factura':  datetime.strptime(factura['FeDetReq']['FECAEDetRequest']['CbtesAsoc'][0]['CbteAsoc']['CbteFch'], '%Y%m%d')
            }
        }
    return factura_formateada

def realizar_facturacion_afip(nro_tipo_comprobante:int,
                              price:float,
                              boleta:dict,
                              merchant:dict,
                              afip:object,
                              where_insert:str,
                              porcentaje_iva:float,
                              MAKE_UPDATE:bool,
                              MAKE_PDF:bool,
                              files_names:dict,
                              factura_existente=None):
    db = init_mongo()
    #### 1) Obtiene el ultimo nro de cbte segun el tipo de cbte
    ultimo_comprobante = afip.ultimoAutorizado(nro_tipo_comprobante, merchant)
    nro_comprobante = ultimo_comprobante['CbteNro'] if ultimo_comprobante else 0
    muvi_print('info', f"Nro de ultimo comprobante: {nro_comprobante} para el tipo {nro_tipo_comprobante}")

    #### 2) Setea el formato de la factura
    factura_afip = format_factura_afip(boleta=boleta,
                                       merchant=merchant,
                                       nro_comprobante=nro_comprobante+1,
                                       tipo_cbte=nro_tipo_comprobante,
                                       price=price,
                                       porcentaje_iva=porcentaje_iva,
                                       factura_existente=factura_existente)
    pprint(factura_afip)
    #### 3) Intenta autorizar con AFIP
    try:
        response_afip = afip.autorizarComprobante(factura_afip,merchant)
        print(response_afip)
    except:
        print(traceback.format_exc())
        muvi_print('error', 'ERROR: No se pudo generar el diccionario de factura')
        return ["error", "ERROR: Fallo la autorizacion por parte de AFIP"]

    #### 4) Si se facturo, se procede a realizar los MAKE_UPDATE y MAKE_PDF

    if 'FeCabResp' in response_afip and response_afip['FeCabResp'] and response_afip['FeCabResp']['Resultado'] == 'A':
        factura_db = format_factura_db(boleta=boleta,
                                       factura=factura_afip,
                                       response_factura=response_afip,
                                       data_afip=merchant['afip'])
        if MAKE_UPDATE:
            print('Insertando la factura en la collection de facturas...')
            insert_factura = db.facturas.insert_one(factura_db)
            if insert_factura.acknowledged:
                muvi_print('successful', f"Se inserto una factura con el id {insert_factura.inserted_id}")
                data_update_boleta = {where_insert:insert_factura.inserted_id}
                print('Se inserta en la boleta:', data_update_boleta)
                result_update_boleta = db.boletas.update_one({'_id': boleta['_id']}, {'$set': data_update_boleta})
                muvi_print('info',f"Se encontro {result_update_boleta.matched_count} y se actualizo {result_update_boleta.modified_count}")
                if result_update_boleta.modified_count == 0:
                    return ['error','error_update_boleta']
            else:
                return ["error","error_insert_factura"]
        if MAKE_PDF:
            print('Generando pdf...')
            files_names['PDF_NAME'] = make_name_pdf(factura_db)
            pdf_filename = make_factura_pdf(files_names=files_names,factura=factura_db)
            muvi_print('successful','PDF generado con exito')
        else:
            pdf_filename = 'error_not_make_pdf'
    else:
        return ["error", "ERROR: Fallo la autorizacion por parte de AFIP"]

    return ['successful',pdf_filename,insert_factura.inserted_id]